@@ -34,26 +34,26 @@ namespace eval ttk {
 
 ### Option database settings.
 #
-option add *TEntry.cursor [ttk::cursor text]
+option add *TEntry.cursor [ttk::cursor text] widgetDefault
 
 ### Bindings.
 #
 # Removed the following standard Tk bindings:
 #
-# <Control-Key-space>, <Control-Shift-Key-space>,
-# <Key-Select>,  <Shift-Key-Select>:
-#	ttk::entry widget doesn't use selection anchor.
-# <Key-Insert>:
+# <Control-space>, <Control-Shift-space>,
+# <Select>,  <Shift-Select>:
+#	Ttk entry widget doesn't use selection anchor.
+# <Insert>:
 #	Inserts PRIMARY selection (on non-Windows platforms).
 #	This is inconsistent with typical platform bindings.
-# <Double-Shift-ButtonPress-1>, <Triple-Shift-ButtonPress-1>:
+# <Double-Shift-Button-1>, <Triple-Shift-Button-1>:
 #	These don't do the right thing to start with.
-# <Meta-Key-b>, <Meta-Key-d>, <Meta-Key-f>,
-# <Meta-Key-BackSpace>, <Meta-Key-Delete>:
+# <Meta-b>, <Meta-d>, <Meta-f>,
+# <Meta-BackSpace>, <Meta-Delete>:
 #	Judgment call.  If <Meta> happens to be assigned to the Alt key,
 #	these could conflict with application accelerators.
 #	(Plus, who has a Meta key these days?)
-# <Control-Key-t>:
+# <Control-t>:
 #	Another judgment call.  If anyone misses this, let me know
 #	and I'll put it back.
 #
@@ -68,86 +68,109 @@ bind TEntry <<Clear>> 			{ ttk::entry::Clear %W }
 ## Button1 bindings:
 #	Used for selection and navigation.
 #
-bind TEntry <ButtonPress-1> 		{ ttk::entry::Press %W %x }
-bind TEntry <Shift-ButtonPress-1>	{ ttk::entry::Shift-Press %W %x }
-bind TEntry <Double-ButtonPress-1> 	{ ttk::entry::Select %W %x word }
-bind TEntry <Triple-ButtonPress-1> 	{ ttk::entry::Select %W %x line }
+bind TEntry <Button-1> 			{ ttk::entry::Press %W %x }
+bind TEntry <Shift-Button-1>		{ ttk::entry::Shift-Press %W %x }
+bind TEntry <Double-Button-1> 		{ ttk::entry::Select %W %x word }
+bind TEntry <Triple-Button-1> 		{ ttk::entry::Select %W %x line }
 bind TEntry <B1-Motion>			{ ttk::entry::Drag %W %x }
 
-bind TEntry <B1-Leave> 		{ ttk::entry::DragOut %W %m }
-bind TEntry <B1-Enter>		{ ttk::entry::DragIn %W }
-bind TEntry <ButtonRelease-1>	{ ttk::entry::Release %W }
+bind TEntry <B1-Leave> 			{ ttk::entry::DragOut %W %m }
+bind TEntry <B1-Enter>			{ ttk::entry::DragIn %W }
+bind TEntry <ButtonRelease-1>		{ ttk::entry::Release %W }
 
-bind TEntry <Control-ButtonPress-1> {
+bind TEntry <<ToggleSelection>> {
     %W instate {!readonly !disabled} { %W icursor @%x ; focus %W }
 }
 
-## Button2 bindings:
+## Button2 (Button3 on Aqua) bindings:
 #	Used for scanning and primary transfer.
-#	Note: ButtonRelease-2 is mapped to <<PasteSelection>> in tk.tcl.
+#	Note: ButtonRelease-2 (ButtonRelease-3 on Aqua)
+#	is mapped to <<PasteSelection>> in tk.tcl.
 #
-bind TEntry <ButtonPress-2> 		{ ttk::entry::ScanMark %W %x }
-bind TEntry <B2-Motion> 		{ ttk::entry::ScanDrag %W %x }
-bind TEntry <ButtonRelease-2>		{ ttk::entry::ScanRelease %W %x }
+if {[tk windowingsystem] ne "aqua"} {
+    bind TEntry <Button-2> 		{ ttk::entry::ScanMark %W %x }
+    bind TEntry <B2-Motion> 		{ ttk::entry::ScanDrag %W %x }
+    bind TEntry <ButtonRelease-2>	{ ttk::entry::ScanRelease %W %x }
+} else {
+    bind TEntry <Button-3> 		{ ttk::entry::ScanMark %W %x }
+    bind TEntry <B3-Motion> 		{ ttk::entry::ScanDrag %W %x }
+    bind TEntry <ButtonRelease-3>	{ ttk::entry::ScanRelease %W %x }
+}
 bind TEntry <<PasteSelection>>		{ ttk::entry::ScanRelease %W %x }
 
 ## Keyboard navigation bindings:
 #
-bind TEntry <Key-Left> 			{ ttk::entry::Move %W prevchar }
-bind TEntry <Key-Right> 		{ ttk::entry::Move %W nextchar }
-bind TEntry <Control-Key-Left>		{ ttk::entry::Move %W prevword }
-bind TEntry <Control-Key-Right>		{ ttk::entry::Move %W nextword }
-bind TEntry <Key-Home>			{ ttk::entry::Move %W home }
-bind TEntry <Key-End>			{ ttk::entry::Move %W end }
+bind TEntry <<PrevChar>>		{ ttk::entry::Move %W prevchar }
+bind TEntry <<NextChar>> 		{ ttk::entry::Move %W nextchar }
+bind TEntry <<PrevWord>>		{ ttk::entry::Move %W prevword }
+bind TEntry <<NextWord>>		{ ttk::entry::Move %W nextword }
+bind TEntry <<LineStart>>		{ ttk::entry::Move %W home }
+bind TEntry <<LineEnd>>			{ ttk::entry::Move %W end }
 
-bind TEntry <Shift-Key-Left> 		{ ttk::entry::Extend %W prevchar }
-bind TEntry <Shift-Key-Right>		{ ttk::entry::Extend %W nextchar }
-bind TEntry <Shift-Control-Key-Left>	{ ttk::entry::Extend %W prevword }
-bind TEntry <Shift-Control-Key-Right>	{ ttk::entry::Extend %W nextword }
-bind TEntry <Shift-Key-Home>		{ ttk::entry::Extend %W home }
-bind TEntry <Shift-Key-End>		{ ttk::entry::Extend %W end }
+bind TEntry <<SelectPrevChar>> 		{ ttk::entry::Extend %W prevchar }
+bind TEntry <<SelectNextChar>>		{ ttk::entry::Extend %W nextchar }
+bind TEntry <<SelectPrevWord>>		{ ttk::entry::Extend %W prevword }
+bind TEntry <<SelectNextWord>>		{ ttk::entry::Extend %W nextword }
+bind TEntry <<SelectLineStart>>		{ ttk::entry::Extend %W home }
+bind TEntry <<SelectLineEnd>>		{ ttk::entry::Extend %W end }
 
-bind TEntry <Control-Key-slash> 	{ %W selection range 0 end }
-bind TEntry <Control-Key-backslash> 	{ %W selection clear }
+bind TEntry <<SelectAll>> 		{ %W selection range 0 end }
+bind TEntry <<SelectNone>> 		{ %W selection clear }
 
 bind TEntry <<TraverseIn>> 	{ %W selection range 0 end; %W icursor end }
 
 ## Edit bindings:
 #
-bind TEntry <KeyPress> 			{ ttk::entry::Insert %W %A }
-bind TEntry <Key-Delete>		{ ttk::entry::Delete %W }
-bind TEntry <Key-BackSpace> 		{ ttk::entry::Backspace %W }
+bind TEntry <Key> 			{ ttk::entry::Insert %W %A }
+bind TEntry <Delete>			{ ttk::entry::Delete %W }
+bind TEntry <BackSpace> 		{ ttk::entry::Backspace %W }
 
 # Ignore all Alt, Meta, and Control keypresses unless explicitly bound.
-# Otherwise, the <KeyPress> class binding will fire and insert the character.
+# Otherwise, the <Key> class binding will fire and insert the character.
 # Ditto for Escape, Return, and Tab.
 #
-bind TEntry <Alt-KeyPress>		{# nothing}
-bind TEntry <Meta-KeyPress>		{# nothing}
-bind TEntry <Control-KeyPress> 		{# nothing}
-bind TEntry <Key-Escape> 		{# nothing}
-bind TEntry <Key-Return> 		{# nothing}
-bind TEntry <Key-KP_Enter> 		{# nothing}
-bind TEntry <Key-Tab> 			{# nothing}
+bind TEntry <Alt-Key>			{# nothing}
+bind TEntry <Meta-Key>			{# nothing}
+bind TEntry <Control-Key> 		{# nothing}
+bind TEntry <Escape> 			{# nothing}
+bind TEntry <Return> 			{# nothing}
+bind TEntry <KP_Enter> 			{# nothing}
+bind TEntry <Tab> 			{# nothing}
 
 # Argh.  Apparently on Windows, the NumLock modifier is interpreted
 # as a Command modifier.
 if {[tk windowingsystem] eq "aqua"} {
-    bind TEntry <Command-KeyPress>	{# nothing}
+    bind TEntry <Command-Key>		{# nothing}
+    bind TEntry <Mod4-Key>		{# nothing}
 }
 # Tk-on-Cocoa generates characters for these two keys. [Bug 2971663]
-bind TEntry <Down>			{# nothing}
-bind TEntry <Up>			{# nothing}
+bind TEntry <<PrevLine>>		{# nothing}
+bind TEntry <<NextLine>>		{# nothing}
 
 ## Additional emacs-like bindings:
 #
-bind TEntry <Control-Key-a>		{ ttk::entry::Move %W home }
-bind TEntry <Control-Key-b>		{ ttk::entry::Move %W prevchar }
-bind TEntry <Control-Key-d> 		{ ttk::entry::Delete %W }
-bind TEntry <Control-Key-e> 		{ ttk::entry::Move %W end }
-bind TEntry <Control-Key-f> 		{ ttk::entry::Move %W nextchar }
-bind TEntry <Control-Key-h>		{ ttk::entry::Backspace %W }
-bind TEntry <Control-Key-k>		{ %W delete insert end }
+bind TEntry <Control-d>			{ ttk::entry::Delete %W }
+bind TEntry <Control-h>			{ ttk::entry::Backspace %W }
+bind TEntry <Control-k>			{ %W delete insert end }
+
+# Bindings for IME text input.
+
+bind TEntry <<TkStartIMEMarkedText>> {
+    dict set ::tk::Priv(IMETextMark) "%W" [%W index insert]
+}
+bind TEntry <<TkEndIMEMarkedText>> {
+    if { [catch {dict get $::tk::Priv(IMETextMark) "%W"} mark] } {
+	bell
+    } else {
+	%W selection range $mark insert
+    }
+}
+bind TEntry <<TkClearIMEMarkedText>> {
+    %W delete [dict get $::tk::Priv(IMETextMark) "%W"] [%W index insert]
+}
+bind TEntry <<TkAccentBackspace>> {
+    ttk::entry::Backspace %W
+}
 
 ### Clipboard procedures.
 #
@@ -215,7 +238,6 @@ proc ttk::entry::ClosestGap {w x} {
 ## See $index -- Make sure that the character at $index is visible.
 #
 proc ttk::entry::See {w {index insert}} {
-    update idletasks	;# ensure scroll data up-to-date
     set c [$w index $index]
     # @@@ OR: check [$w index left] / [$w index right]
     if {$c < [$w index @0] || $c >= [$w index @[winfo width $w]]} {
@@ -232,6 +254,10 @@ set ::ttk::entry::State(startNext) \
 	[string equal [tk windowingsystem] "win32"]
 
 proc ttk::entry::NextWord {w start} {
+    # the check on [winfo class] is because the spinbox and combobox also use this proc
+    if {[winfo class $w] eq "TEntry" && [$w cget -show] ne ""} {
+	return end
+    }
     variable State
     set pos [tcl_endOfWord [$w get] [$w index $start]]
     if {$pos >= 0 && $State(startNext)} {
@@ -246,6 +272,10 @@ proc ttk::entry::NextWord {w start} {
 ## PrevWord -- Find the previous word position.
 #
 proc ttk::entry::PrevWord {w start} {
+    # the check on [winfo class] is because the spinbox and combobox also use this proc
+    if {[winfo class $w] eq "TEntry" && [$w cget -show] ne ""} {
+	return 0
+    }
     set pos [tcl_startOfPreviousWord [$w get] [$w index $start]]
     if {$pos < 0} {
 	return 0
@@ -337,7 +367,7 @@ proc ttk::entry::Extend {w where} {
 # Triple-clicking enters "line-select" mode.
 #
 
-## Press -- ButtonPress-1 binding.
+## Press -- Button-1 binding.
 #	Set the insertion cursor, claim the input focus, set up for
 #	future drag operations.
 #
@@ -354,7 +384,7 @@ proc ttk::entry::Press {w x} {
     set State(anchor) [$w index insert]
 }
 
-## Shift-Press -- Shift-ButtonPress-1 binding.
+## Shift-Press -- Shift-Button-1 binding.
 #	Extends the selection, sets anchor for future drag operations.
 #
 proc ttk::entry::Shift-Press {w x} {
@@ -422,7 +452,7 @@ proc ttk::entry::DragOut {w mode} {
 # 	Suspend autoscroll.
 #
 proc ttk::entry::DragIn {w} {
-    ttk::CancelRepeat 
+    ttk::CancelRepeat
 }
 
 ## <ButtonRelease-1> binding
@@ -436,7 +466,7 @@ proc ttk::entry::Release {w} {
 ## AutoScroll
 #	Called repeatedly when the mouse is outside an entry window
 #	with Button 1 down.  Scroll the window left or right,
-#	depending on where the mouse left the window, and extend 
+#	depending on where the mouse left the window, and extend
 #	the selection according to the current selection mode.
 #
 # TODO: AutoScroll should repeat faster (50ms) than normal autorepeat.
@@ -503,7 +533,7 @@ proc ttk::entry::LineSelect {w _ _} {
 ### Button 2 binding procedures.
 #
 
-## ScanMark -- ButtonPress-2 binding.
+## ScanMark -- Button-2 binding.
 #	Marks the start of a scan or primary transfer operation.
 #
 proc ttk::entry::ScanMark {w x} {
