@@ -9,15 +9,15 @@ namespace eval ttk::scale {
     }
 }
 
-bind TScale <ButtonPress-1>   { ttk::scale::Press %W %x %y }
+bind TScale <Button-1>        { ttk::scale::Press %W %x %y }
 bind TScale <B1-Motion>       { ttk::scale::Drag %W %x %y }
 bind TScale <ButtonRelease-1> { ttk::scale::Release %W %x %y }
 
-bind TScale <ButtonPress-2>   { ttk::scale::Jump %W %x %y }
+bind TScale <Button-2>        { ttk::scale::Jump %W %x %y }
 bind TScale <B2-Motion>       { ttk::scale::Drag %W %x %y }
 bind TScale <ButtonRelease-2> { ttk::scale::Release %W %x %y }
 
-bind TScale <ButtonPress-3>   { ttk::scale::Jump %W %x %y }
+bind TScale <Button-3>        { ttk::scale::Jump %W %x %y }
 bind TScale <B3-Motion>       { ttk::scale::Drag %W %x %y }
 bind TScale <ButtonRelease-3> { ttk::scale::Release %W %x %y }
 
@@ -42,7 +42,7 @@ proc ttk::scale::Press {w x y} {
     switch -glob -- [$w identify $x $y] {
 	*track -
         *trough {
-            set inc [expr {([$w get $x $y] <= [$w get]) ? -1 : 1}]
+            set inc [expr {([$w get $x $y] <= [$w get]) ^ ([$w cget -from] > [$w cget -to]) ? -1 : 1}]
             ttk::Repeatedly Increment $w $inc
         }
         *slider {
@@ -52,7 +52,7 @@ proc ttk::scale::Press {w x y} {
     }
 }
 
-# scale::Jump -- ButtonPress-2/3 binding for scale acts like
+# scale::Jump -- Button-2/3 binding for scale acts like
 #	Press except that clicking in the trough jumps to the
 #	clicked position.
 proc ttk::scale::Jump {w x y} {
@@ -87,5 +87,8 @@ proc ttk::scale::Release {w x y} {
 
 proc ttk::scale::Increment {w delta} {
     if {![winfo exists $w]} return
+    if {([$w cget -from] > [$w cget -to])} {
+	set delta [expr {-$delta}]
+    }
     $w set [expr {[$w get] + $delta}]
 }
